/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================                                                                               
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

/**
*  \file 
*  \brief Class bbtk::GObjectsMVCFactory 
*/


#include "GObjectsMVCFactory.h"

namespace bbtk
{


	//=========================================================================

	GObjectsMVCFactory* GObjectsMVCFactory:: instance = NULL;
	GObjectsMVCFactory::GObjectsMVCFactory()
	{
	}

	//=========================================================================
	GObjectsMVCFactory::~GObjectsMVCFactory()
	{
		GObjectsMVCFactory::destroyInstance();
	}
	//=========================================================================

		
	//=========================================================================
	GObjectsMVCFactory* GObjectsMVCFactory :: getInstance()
	{
		if(instance == NULL){
			instance = new GObjectsMVCFactory();
		}
		return instance;
	}

	//=========================================================================

	void GObjectsMVCFactory :: destroyInstance()
	{
		//FCY memory leaks
		delete instance;
		instance=NULL;
	}

	//=========================================================================

	vtkGObjectView* GObjectsMVCFactory :: createGObjectView(int type)
	{
		vtkGObjectView* view = NULL;
		if (type == GBLACKBOX)
		{
			view = new vtkGBlackBoxView();
		}
		else if(type == GPORT)
		{
			view = new vtkGPortView();
		}
		else if(type == GCOMPLEXINPUTPORT || type == GCOMPLEXOUTPUTPORT)
		{
			view = new vtkGComplexBoxPortView();
		}
		return view;
	}

	//=========================================================================

	GObjectController* GObjectsMVCFactory :: createGObjectController(int type)
	{
		GObjectController *controller = NULL;
		if (type == GBLACKBOX)
		{
			controller = new GBlackBoxController();
		}
		else if(type == GPORT)
		{
			controller = new GPortController();
		}
		else if(type == GCOMPLEXINPUTPORT || type == GCOMPLEXOUTPUTPORT)
		{
			controller = new GBoxController();
		}
		return controller;
	}

	//=========================================================================
	
	GObjectModel* GObjectsMVCFactory :: createGObjectModel(int type)
	{
		GObjectModel *model = NULL;
		if (type == GBLACKBOX)
		{
			model = new GBlackBoxModel();
		}
		else if(type == GPORT)
		{
			model = new GPortModel();
		}
		else if(type == GCOMPLEXINPUTPORT || type == GCOMPLEXOUTPUTPORT)
		{
			model = new GComplexBoxPortModel();
		}

		model->setGObjectType(type);

		return model;
	}

	//=========================================================================


}  // EO namespace bbtk

// EOF

